@@ -1,5 +1,5 @@
 # -* coding: utf-8 -*-
-# Copyright (c) 2011 gocept gmbh & co. kg
+# Copyright (c) 2011-2012 gocept gmbh & co. kg
 # See also LICENSE.txt
 
 from Products.ZReST.ZReST import manage_addZReST
@@ -54,6 +54,7 @@ class ViewTests(Testing.ZopeTestCase.FunctionalTestCase):
         self.app['acl_users']._doAddUser('manager', 'asdf', ('Manager',), [])
 
     def test_checkout_response_should_be_OK_and_a_snarf_archive(self):
+        # XXX flaky, see #10491
         browser = zope.testbrowser.browser.Browser()
         browser.addHeader('Authorization',
                           'Basic '+'manager:asdf'.encode('base64'))
@@ -311,12 +312,46 @@ class FolderTest(Testing.ZopeTestCase.FunctionalTestCase):
         ignore.source = 'bar\nbaz'
         response = self.publish(
             '/folder/@@toFS.snarf', basic='manager:asdf')
-        self.assertEquals("""\
+        self.assertEqual("""\
   <entry name="foo"
   <entry name="fssync-dump-ignore"
 """, grep('<entry', unsnarf(response, 'folder/@@Zope/Entries.xml'),
           sort=True))
 
+    def test_items_in_ignore_file_are_kept_on_load(self):
+        self.app.manage_addFolder('folder')
+        folder = self.app['folder']
+        folder.manage_addFile('foo', 'one')
+        folder.manage_addFile('bar', 'two')
+        manage_addZReST(folder, 'fssync-dump-ignore')
+        ignore = folder['fssync-dump-ignore']
+        ignore.source = 'bar'
+
+        folder.manage_addFolder('subfolder')
+        subfolder = folder['subfolder']
+        subfolder.manage_addFile('baz', 'one')
+        subfolder.manage_addFile('qux', 'two')
+        manage_addZReST(subfolder, 'fssync-dump-ignore')
+        ignore = subfolder['fssync-dump-ignore']
+        ignore.source = 'qux'
+
+        app_object_ids = sorted(self.app.objectIds())
+
+        response = self.publish(
+            '/folder/@@toFS.snarf', basic='manager:asdf')
+        self.publish('/@@checkin.snarf?note=test&name=folder&src=folder',
+                     basic='manager:asdf',
+                     request_method='POST',
+                     env={'CONTENT_TYPE': 'application/x-snarf'},
+                     stdin=StringIO.StringIO(response.getBody()),
+                     handle_errors=False)
+
+        self.assertEqual(app_object_ids, sorted(self.app.objectIds()))
+        self.assertEqual(['bar', 'foo', 'fssync-dump-ignore', 'subfolder'],
+                         sorted(self.app['folder'].objectIds()))
+        self.assertEqual(['baz', 'fssync-dump-ignore', 'qux'],
+                         sorted(self.app['folder']['subfolder'].objectIds()))
+
 
 class PythonScriptTest(Testing.ZopeTestCase.FunctionalTestCase):
     """Make sure leaving out compiled code doesn't break anything.
@@ -531,6 +566,7 @@ class BaseFileSystemTests(Testing.ZopeTestCase.FunctionalTestCase):
         # save host and credentials
         self.host = 'localhost:%s' % self.layer.port
         self.credentials = 'manager:asdf'
+        transaction.commit()
 
     def tearDown(self):
         shutil.rmtree(self.repository)
@@ -556,7 +592,6 @@ class UserFolderTest(BaseFileSystemTests):
     def test_userfolder_is_not_duplicated_after_checkout_checkin(self):
         self.app.manage_addFolder('folder')
         self.app['folder'].manage_addProduct['OFSP'].manage_addUserFolder()
-        import transaction
         transaction.commit()
         self.assertTrue(self.app['folder'].__allow_groups__.aq_base is
                         self.app['folder']['acl_users'].aq_base)
@@ -722,7 +757,7 @@ class RoundTripTest(BaseFileSystemTests):
             self.host, 'base', self.credentials, self.repository)
         # delete foo in base
         self.app['base'].manage_delObjects('foo')
-        # update repository
+        # update the repository to get rid of out-of-sync errors
         gocept.fssyncz2.main.dump(
             self.host, 'base', self.credentials, self.repository)
 
@@ -731,7 +766,6 @@ class RoundTripTest(BaseFileSystemTests):
 
         # foo is back in database and should be removed now
         self.assertEquals(self.app['base']['foo'].data, 'Content of foo')
-        # update the repository to get rid of out-of-sync errors
         gocept.fssyncz2.main.load(
             self.host, 'base', self.credentials, self.repository)
         # foo is now removed
